"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.deactivate = exports.activate = void 0;
// The module 'vscode' contains the VS Code extensibility API
// Import the module and reference it with the alias vscode in your code below
const vscode = __importStar(require("vscode"));
const vscode_1 = require("vscode");
const statusBar_1 = require("./statusBar");
const recorder_1 = require("./recorder");
const config_1 = require("./config");
// This method is called when your extension is activated
// Your extension is activated the very first time the command is executed
function activate(context) {
    const config = new config_1.CherryConfigModel(context);
    const statusBar = new statusBar_1.StatusBar();
    let runner = null;
    context.subscriptions.push(statusBar);
    // Use the console to output diagnostic information (console.log) and errors (console.error)
    // This line of code will only be executed once when your extension is activated
    console.log('Congratulations, your extension "cherry-studio-recorder" is now active!');
    vscode.commands.registerCommand('cherry-studio-recorder.start', start);
    vscode.commands.registerCommand('cherry-studio-recorder.stop', stop);
    async function start() {
        if (statusBar.status() !== "ready") {
            return; // prevent from double click issue
        }
        const foldersUri = await vscode.window.showOpenDialog({
            canSelectFiles: false,
            canSelectFolders: true,
            canSelectMany: false,
            openLabel: "Select folder",
            defaultUri: vscode.Uri.file(config.recentDir)
        });
        if (!foldersUri) {
            vscode.window.showInformationMessage('[Cherry studio] Action canceled');
            return;
        }
        if (foldersUri.length > 1) {
            vscode.window.showInformationMessage('[Cherry studio] Action failed: multiple folders are not allowed');
            return;
        }
        const componentName = await vscode.window.showInputBox({
            title: "Start recording",
            prompt: "Component name",
            placeHolder: "Enter component name showing in Cherry studio dashboard",
            value: await vscode.env.clipboard.readText()
        });
        if (!componentName) {
            vscode.window.showInformationMessage('[Cherry studio] Action failed: component name empty');
            return;
        }
        const sessionFolder = foldersUri[0];
        const componentFileUri = vscode_1.Uri.joinPath(sessionFolder, `${componentName}`);
        await vscode.workspace.fs.writeFile(componentFileUri, new Uint8Array());
        statusBar.starting();
        recorder_1.Recorder.run(config, sessionFolder, componentName, componentFileUri).then((r) => {
            runner = r;
            statusBar.running();
            runner.finish.then(() => {
                statusBar.ready();
            });
        });
        config.recentDir = sessionFolder.path;
    }
    function stop() {
        if (statusBar.status() !== "running") {
            return; // prevent from double click issue
        }
        statusBar.stopping();
        runner?.stop();
    }
}
exports.activate = activate;
// This method is called when your extension is deactivated
function deactivate() { }
exports.deactivate = deactivate;
//# sourceMappingURL=extension.js.map