"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Recorder = void 0;
const vscode = __importStar(require("vscode"));
const vscode_1 = require("vscode");
const zipper_1 = require("./utils/zipper");
const string_1 = require("./utils/string");
const file_1 = require("./utils/file");
class Recorder {
    static async run(config, sessionFolder, componentNameDraft, componentFileDraft) {
        let isTest = false;
        let stopped = false;
        let stopping = false;
        let filePathList = [];
        let durationInSeconds = 0;
        let changes = [];
        const wsIds = [];
        const changeIds = [];
        const componentName = componentFileDraft.path.replace(".draft", "");
        const componentFile = vscode.Uri.file(componentName);
        {
            const folderPathParts = sessionFolder.path.split("/");
            if (folderPathParts[folderPathParts.length - 1] === "test") { // test session
                isTest = true;
            }
        }
        /**
         * Backup current workspace
         */
        // for each workspace, zip whole folder
        const zipper = new zipper_1.Zipper(config);
        for (const ws of vscode.workspace.workspaceFolders ?? []) {
            const [fpl, content] = await zipper.zipFolder(ws.uri);
            filePathList = fpl;
            let wsId = (0, string_1.generateRandomString)(10);
            wsIds.push({
                path: ws.uri.path,
                id: wsId,
                length: content.length
            });
            await vscode.workspace.fs.writeFile(vscode.Uri.joinPath(sessionFolder, `${componentNameDraft}.${wsId}`), content);
            changes.push({
                type: "add_workspace",
                path: ws.uri.path,
                startFromSecond: durationInSeconds,
            });
        }
        /**
         * Interval jobs
         */
        let durationTimer;
        let processChangeTimer;
        let watchComponentFileTimer;
        durationTimer = setInterval(() => {
            durationInSeconds += 0.2;
        }, 200);
        processChangeTimer = setInterval(async () => {
            // process changes
            if (stopping || stopped) {
                return; // for safe, never happen
            }
            if (changes.length > 30) {
                const markedLength = changes.length;
                let changeId = (0, string_1.generateRandomString)(10);
                const removeChanges = changes.splice(0, markedLength);
                const content = Buffer.from(JSON.stringify(removeChanges));
                changeIds.push({
                    id: changeId,
                    length: content.length
                });
                await vscode.workspace.fs.writeFile(vscode.Uri.joinPath(sessionFolder, `${componentNameDraft}.${changeId}`), content);
            }
        }, 30000);
        watchComponentFileTimer = setInterval(async () => {
            // process changes
            if (stopping || stopped) {
                return; // for safe, never happen
            }
            try {
                await vscode.workspace.fs.stat(vscode.Uri.file(componentName));
                stop();
            }
            catch (error) {
                // do nothing
            }
        }, 1000);
        /**
         * Listener
         */
        const watcher = vscode.workspace.createFileSystemWatcher('**/*');
        // If we don't have this, file created outside workspace won't be caught by vscode extension
        // when we have this watcher, vscode.workspace.onDidOpenTextDocument event will be fired when new file is created.
        watcher.onDidCreate(async (e) => {
            const f = await vscode.workspace.fs.stat(e);
            if (f.type === vscode_1.FileType.File) {
                changes.push({
                    type: "create_file",
                    path: e.path,
                    startFromSecond: durationInSeconds,
                    content: (await vscode.workspace.fs.readFile(e)).toString()
                });
            }
        });
        watcher.onDidChange(async (e) => {
            const f = await vscode.workspace.fs.stat(e);
            const fileOpened = vscode.workspace.textDocuments.filter(d => d.uri.path === e.path && !d.isClosed).length > 0;
            // fileOpened means it's changed from editor, it will be tracked by onDidChangeTextDocument event.
            if (f.type === vscode_1.FileType.File && !fileOpened) {
                changes.push({
                    type: "create_file",
                    path: e.path,
                    startFromSecond: durationInSeconds,
                    content: (await vscode.workspace.fs.readFile(e)).toString()
                });
            }
        });
        const listener = [];
        listener.push(vscode.workspace.onDidOpenTextDocument(async (doc) => {
            if (stopping || stopped) {
                return; // for safe, never happen
            }
            if (filePathList.indexOf(doc.uri.path) < 0) {
                changes.push({
                    type: "create_file",
                    path: doc.uri.path,
                    startFromSecond: durationInSeconds,
                    content: (await vscode.workspace.fs.readFile(doc.uri)).toString(),
                });
                filePathList.push(doc.uri.path);
            }
            changes.push({
                type: "open_file",
                path: doc.uri.path,
                startFromSecond: durationInSeconds
            });
        }));
        listener.push(vscode.workspace.onDidChangeTextDocument(async (textChangeEvent) => {
            if (stopping || stopped) {
                return; // for safe, never happen
            }
            for (const change of textChangeEvent.contentChanges) {
                if (filePathList.indexOf(textChangeEvent.document.uri.path) < 0) {
                    changes.push({
                        type: "create_file",
                        path: textChangeEvent.document.uri.path,
                        startFromSecond: durationInSeconds,
                        content: (await vscode.workspace.fs.readFile(textChangeEvent.document.uri)).toString(),
                    });
                    filePathList.push(textChangeEvent.document.uri.path);
                }
                changes.push({
                    type: "change_file_content",
                    path: textChangeEvent.document.uri.path,
                    changedText: change.text,
                    startFromSecond: durationInSeconds,
                    range: {
                        ec: change.range.start.character, // endColumn
                        eln: change.range.end.line, // endLineNumber
                        sc: change.range.end.character, // startColumn
                        sln: change.range.start.line, // startLineNumber
                    }
                });
            }
        }));
        listener.push(vscode.workspace.onDidCreateFiles((fileCreateEvent) => {
            if (stopping || stopped) {
                return; // for safe, never happen
            }
            fileCreateEvent.files.map(async (e) => {
                const file = await vscode.workspace.fs.stat(vscode.Uri.parse(e.path));
                const isDirectory = file.type === vscode_1.FileType.Directory;
                if (isDirectory) {
                    changes.push({
                        type: "create_folder",
                        path: e.path,
                        startFromSecond: durationInSeconds,
                    });
                }
                else {
                    const fContent = (await vscode.workspace.fs.readFile(e)).toString();
                    changes.push({
                        type: "create_file",
                        path: e.path,
                        content: fContent,
                        startFromSecond: durationInSeconds,
                    });
                    filePathList.push(e.path);
                }
            });
        }));
        listener.push(vscode.workspace.onDidRenameFiles(fileRenameEvent => {
            if (stopping || stopped) {
                return; // for safe, never happen
            }
            fileRenameEvent.files.map(async (e) => {
                const file = await vscode.workspace.fs.stat(vscode.Uri.parse(e.newUri.path));
                const isDirectory = file.type === vscode_1.FileType.Directory;
                if (isDirectory) {
                    changes.push({
                        type: "rename_folder",
                        fromPath: e.oldUri.path,
                        newPath: e.newUri.path,
                        startFromSecond: durationInSeconds,
                    });
                }
                else {
                    changes.push({
                        type: "rename_file",
                        fromPath: e.oldUri.path,
                        newPath: e.newUri.path,
                        startFromSecond: durationInSeconds,
                    });
                    const idx = filePathList.indexOf(e.oldUri.path);
                    if (idx > -1) {
                        filePathList.splice(idx, 1);
                    }
                    filePathList.push(e.newUri.path);
                }
            });
        }));
        listener.push(vscode.workspace.onDidDeleteFiles(async (e) => {
            if (stopping || stopped) {
                return; // for safe, never happen
            }
            for await (const f of e.files) {
                const file = await vscode.workspace.fs.stat(vscode.Uri.parse(f.path));
                const isDirectory = file.type === vscode_1.FileType.Directory;
                if (isDirectory) {
                    changes.push({
                        type: "delete_folder",
                        path: f.path,
                        startFromSecond: durationInSeconds,
                    });
                }
                else {
                    changes.push({
                        type: "delete_file",
                        path: f.path,
                        startFromSecond: durationInSeconds,
                    });
                    const idx = filePathList.indexOf(f.path);
                    if (idx > -1) {
                        filePathList.splice(idx, 1);
                    }
                }
            }
        }));
        listener.push(vscode.workspace.onDidChangeWorkspaceFolders(async (wsChangeEvent) => {
            if (stopping || stopped) {
                return; // for safe, never happen
            }
            stop();
        }));
        const stop = async () => {
            stopping = true;
            clearInterval(durationTimer);
            clearInterval(processChangeTimer);
            clearInterval(watchComponentFileTimer);
            listener.map(l => l.dispose());
            await vscode.workspace.fs.writeFile(componentFileDraft, Buffer.from(""));
            changes.push({
                type: "end",
                startFromSecond: changes[changes.length - 1].startFromSecond + 0.1
            });
            let lastChangeId = (0, string_1.generateRandomString)(10);
            const lastChangeContent = Buffer.from(JSON.stringify(changes));
            changeIds.push({
                id: lastChangeId,
                length: lastChangeContent.length
            });
            await vscode.workspace.fs.writeFile(vscode.Uri.joinPath(sessionFolder, `${componentNameDraft}.${lastChangeId}`), lastChangeContent);
            const fileDesUint8Array = (kind, data) => {
                let encoder = new TextEncoder();
                return encoder.encode(JSON.stringify({
                    kind: kind,
                    data: data,
                }));
            };
            let outContentLength = 0;
            let outContentArrayLike = [];
            {
                for (const { id: wsId, path: wsPath } of wsIds) {
                    const tmp = fileDesUint8Array("workspace", wsPath);
                    outContentArrayLike.push((0, file_1.to4ByteUint8Array)(tmp.length));
                    outContentArrayLike.push(tmp);
                    outContentLength += 4 + tmp.length;
                    const uri = vscode.Uri.joinPath(sessionFolder, `${componentNameDraft}.${wsId}`);
                    const f = await vscode.workspace.fs.readFile(uri);
                    outContentArrayLike.push((0, file_1.to4ByteUint8Array)(f.length));
                    outContentArrayLike.push(f);
                    outContentLength += 4 + f.length;
                    await vscode.workspace.fs.delete(uri);
                }
                for (const { id, length } of changeIds) {
                    const tmp = fileDesUint8Array("changes", id);
                    outContentArrayLike.push((0, file_1.to4ByteUint8Array)(tmp.length));
                    outContentArrayLike.push(tmp);
                    outContentLength += 4 + tmp.length;
                    const uri = vscode.Uri.joinPath(sessionFolder, `${componentNameDraft}.${id}`);
                    const f = await vscode.workspace.fs.readFile(uri);
                    outContentArrayLike.push((0, file_1.to4ByteUint8Array)(f.length));
                    outContentArrayLike.push(f);
                    outContentLength += 4 + f.length;
                    await vscode.workspace.fs.delete(uri);
                }
            }
            const outContent = new Uint8Array(outContentLength);
            let offset = 0;
            for (const a of outContentArrayLike) {
                outContent.set(a, offset);
                offset += a.length;
                console.log(`Writing ${a.length} bytes...`);
            }
            if (!isTest) {
                await vscode.workspace.fs.delete(componentFileDraft);
                await vscode.workspace.fs.writeFile(componentFile, outContent);
            }
            stopped = true;
        };
        return {
            stop: stop,
            finish: new Promise((resolve, reject) => {
                setInterval(() => {
                    if (stopped) {
                        resolve();
                    }
                }, 500);
            })
        };
    }
}
exports.Recorder = Recorder;
//# sourceMappingURL=recorder.js.map